import { AppStore, DispatchItem, AnyStoreDeclarer, OperateMode } from "event-flux";
import MultiWinSaver from "./MultiWinSaver";
import { IMainClient, IMainClientCallback, IWinProps } from "./mainClientTypes";
import { DisposableLike } from "event-kit";
interface IWinFilter {
    [storeKey: string]: true | false | IWinFilter;
}
export interface IAppStoreOptions {
    serializer: (value: any) => string;
    deserializer: (str: string) => any;
    MainClient?: IMainClient;
}
export default class MainAppStore extends AppStore implements IMainClientCallback {
    multiWinSaver: MultiWinSaver;
    winFilters: {
        [winId: string]: IWinFilter;
    };
    outStoreDeclarers: string;
    winHoldStores: {
        [winId: string]: Set<string>;
    };
    storeMapModes: {
        [storeMapKey: string]: OperateMode;
    };
    winObserves: {
        [winId: string]: {
            [invokeId: string]: DisposableLike;
        };
    };
    mainClient: IMainClient;
    serializer: (value: any) => string;
    deserializer: (str: string) => any;
    constructor(storeDeclarers?: AnyStoreDeclarer[], options?: IAppStoreOptions);
    init(): this;
    /**
     * The window-spec stores can dependen on the window-spec stores and plain stores
     * The plain stores can only dependen on the plain stores, not window-spec stores.
     */
    _checkStoreRegistry(): void;
    requestStore(storeKey: string, winId: string): DispatchItem;
    releaseStore(storeKey: string, winId: string): void;
    _getStoreKey(storeKey: string, winId: string): string;
    _parseStoreKey(finalStoreKey: string): [string, string];
    _getStateKey(storeKey: string, stateKey: string, winId?: any): string;
    _createStore(storeKey: string, store: DispatchItem, winId: string): void;
    _deleteStore(storeKey: string, winId: string): void;
    _getStoreMethod(storeKey: string, winId: string, method: string, index?: string): any;
    _handleRendererPayload(winId: string, payload: string): Promise<any>;
    handleRendererDispatch(winId: string, invokeId: string, stringifiedAction: string): Promise<void> | undefined;
    handleRendererDispatchNoReturn(winId: string, stringifiedAction: string): void;
    handleRendererDispatchObserve(winId: string, invokeId: string, action: any): void;
    handleRendererDispatchDispose(winId: string, invokeId: string): void;
    handleWinMessage(senderId: string, targetId: string, data: any): void;
    handleRegisterWin(winId: string): void;
    initWin(winId: string, params: IWinProps): void;
    _applyFilterForStore(winFilter: IWinFilter, winId: string, storeKeys: string[], forAdd: boolean): {
        [x: string]: boolean | IWinFilter;
    };
    handleRequestStores(winId: string, storeKeys: string[]): void;
    handleReleaseStores(winId: string, storeKeys: string[]): void;
    handleMapRequestStores(winId: string, storeKey: string, mapKeys: string[]): void;
    handleMapReleaseStores(winId: string, storeKey: string, mapKeys: string[]): void;
    getStoreDeclarers(): string;
    _transformWinState(finalState: any): any;
    getInitStates(winId: string): string;
    handleWillChange(prevState: any, state: any): void;
    forwardState(prevState: any, state: any): void;
    forwardDeltaFilter(winId: string, prevFilter: IWinFilter, newFilter: IWinFilter): void;
}
export {};
//# sourceMappingURL=MainAppStore.d.ts.map