"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var event_kit_1 = require("event-kit");
var MultiWinSaver = /** @class */ (function () {
    function MultiWinSaver() {
        this.winInfos = [];
        this.winIdMap = {};
        this.registerIds = new Set();
        this.emitter = new event_kit_1.Emitter();
    }
    MultiWinSaver.prototype.addWin = function (winInfo) {
        this.winInfos.push(winInfo);
        this.winIdMap[winInfo.winId] = winInfo;
        this.emitter.emit("did-add-win", winInfo.winId);
    };
    MultiWinSaver.prototype.onDidAddWin = function (callback) {
        return this.emitter.on("did-add-win", callback);
    };
    MultiWinSaver.prototype.deleteWin = function (winId) {
        var winInfo = this.winIdMap[winId];
        if (winInfo) {
            var index = this.winInfos.indexOf(winInfo);
            if (index !== -1) {
                this.winInfos.splice(index, 1);
            }
            delete this.winIdMap[winId];
            this.emitter.emit("did-delete-win", winId);
        }
    };
    MultiWinSaver.prototype.onDidDeleteWin = function (callback) {
        return this.emitter.on("did-delete-win", callback);
    };
    MultiWinSaver.prototype.getWinInfo = function (winId) {
        return this.winIdMap[winId];
    };
    MultiWinSaver.prototype.getWinInfos = function () {
        return this.winInfos;
    };
    MultiWinSaver.prototype.findWinInfo = function (callback) {
        return this.winInfos.find(callback);
    };
    MultiWinSaver.prototype.registerWin = function (winId) {
        this.registerIds.add(winId);
        this.emitter.emit("did-register", winId);
    };
    MultiWinSaver.prototype.isRegister = function (winId) {
        return this.registerIds.has(winId);
    };
    MultiWinSaver.prototype.whenRegister = function (winId, callback) {
        if (this.isRegister(winId)) {
            return callback();
        }
        var disposable;
        var onRegister = function (nowId) {
            if (winId === nowId) {
                callback();
                disposable.dispose();
                disposable = undefined;
            }
        };
        disposable = this.emitter.on("did-register", onRegister);
    };
    return MultiWinSaver;
}());
exports.default = MultiWinSaver;
