"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var event_flux_1 = require("event-flux");
var constants_1 = require("./constants");
var MultiWinStore = /** @class */ (function (_super) {
    __extends(MultiWinStore, _super);
    function MultiWinStore(appStore) {
        var _this = _super.call(this, appStore) || this;
        _this.clientIds = [];
        // namedWinId to clientId map
        _this.clientIdNameMap = {};
        // clientId to namedWinId map
        _this.clientNameIdMap = {};
        _this.groupsMap = {};
        _this.multiWinSaver = _this._appStore.multiWinSaver;
        _this.mainClient = _this._appStore.mainClient;
        return _this;
    }
    MultiWinStore.prototype.init = function () {
        var _this = this;
        var winInfos = this.multiWinSaver.getWinInfos();
        if (winInfos.length === 0) {
            // this.mainClient.createWin("mainClient", { path: "/", name: "mainClient", groups: ["main"] }, {});
            var winProps = { path: "/", name: "mainClient", groups: ["main"] };
            this.mainClient.createWin("mainClient", winProps, {});
            this._addWinProps("mainClient", winProps);
        }
        else {
            this.clientIds = winInfos.map(function (item) { return item.winId; });
        }
        this.multiWinSaver.onDidDeleteWin(function (winId) {
            _this._removeClientId(winId);
            if (winId === "mainClient") {
                _this.closeAllWindows();
            }
        });
    };
    // add winProps for clientId
    MultiWinStore.prototype._addWinProps = function (clientId, winProps) {
        this.clientIds.push(clientId);
        this._updateWinProps(clientId, winProps);
    };
    MultiWinStore.prototype._updateWinProps = function (clientId, winProps) {
        if (winProps.name) {
            this.clientNameIdMap[winProps.name] = clientId;
            this.clientIdNameMap[clientId] = winProps.name;
        }
        if (winProps.groups) {
            this.groupsMap[clientId] = winProps.groups;
        }
    };
    MultiWinStore.prototype._parseWinProps = function (winProps, parentId) {
        var parseProps = typeof winProps === "string" ? { path: winProps } : winProps;
        // default window will be distributed to main group
        if (parseProps.groups === undefined) {
            parseProps.groups = ["main"];
        }
        if (parentId) {
            parseProps.parentId = parentId;
        }
        return parseProps;
    };
    MultiWinStore.prototype._removeClientId = function (clientId) {
        var name = this.clientIdNameMap[clientId];
        if (name) {
            delete this.clientNameIdMap[name];
            delete this.clientIdNameMap[clientId];
        }
        delete this.groupsMap[clientId];
        var index = this.clientIds.indexOf(clientId);
        if (index !== -1) {
            this.clientIds.splice(index, 1);
        }
    };
    MultiWinStore.prototype.createWin = function (winProps, params) {
        var clientId = this._genClientId();
        // get url from winProps
        winProps = this._parseWinProps(winProps);
        this.mainClient.createWin(clientId, winProps, params || {});
        this._addWinProps(clientId, winProps);
        return clientId;
    };
    // Create new win if the specific winId is not exists
    MultiWinStore.prototype.createOrOpenWin = function (winName, url, params) {
        // If the window named winName not exists, then we will create new window
        if (!this.clientNameIdMap[winName]) {
            var winProps = typeof url === "string" ? { path: url, name: winName } : __assign({}, url, { name: winName });
            return this.createWin(winProps, params);
        }
        else {
            var clientId = this.clientNameIdMap[winName];
            // this.changeClientAction(clientId, typeof url === "string" ? url : url.path!);
            var winProps = typeof url === "string" ? { path: url } : __assign({}, url);
            winProps.name = winName;
            winProps.parentId = winProps.parentId;
            this.mainClient.changeWin(this.multiWinSaver.getWinInfo(clientId), winProps, params);
            this.activeWin(clientId);
            this._updateWinProps(clientId, winProps);
            return clientId;
        }
    };
    MultiWinStore.prototype.closeWin = function (clientId) {
        var winInfo = this.multiWinSaver.getWinInfo(clientId);
        if (winInfo) {
            this.mainClient.closeWin(winInfo);
        }
    };
    MultiWinStore.prototype.closeWinByName = function (name) {
        var clientId = this.clientNameIdMap[name];
        if (clientId) {
            this.closeWin(clientId);
        }
    };
    MultiWinStore.prototype.closeWinByGroup = function (group) {
        var e_1, _a;
        try {
            for (var _b = __values(this.clientIds), _c = _b.next(); !_c.done; _c = _b.next()) {
                var clientId = _c.value;
                var groups = this.groupsMap[clientId];
                if (groups && groups.indexOf(group) !== -1) {
                    this.closeWin(clientId);
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    MultiWinStore.prototype.activeWin = function (clientId) {
        var winInfo = this.multiWinSaver.getWinInfo(clientId);
        if (winInfo) {
            this.mainClient.activeWin(winInfo);
        }
    };
    MultiWinStore.prototype.activeWindow = function (clientId) {
        this.activeWin(clientId);
    };
    MultiWinStore.prototype.activeWinByName = function (name) {
        var clientId = this.clientNameIdMap[name];
        if (clientId) {
            this.activeWin(clientId);
        }
    };
    MultiWinStore.prototype.activeWinByGroup = function (group) {
        var e_2, _a;
        try {
            for (var _b = __values(this.clientIds), _c = _b.next(); !_c.done; _c = _b.next()) {
                var clientId = _c.value;
                var groups = this.groupsMap[clientId];
                if (groups && groups.indexOf(group) !== -1) {
                    this.activeWin(clientId);
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    MultiWinStore.prototype.sendWinMsg = function (clientId, message) {
        var winInfo = this.multiWinSaver.getWinInfo(clientId);
        if (winInfo) {
            this.mainClient.sendWinMsg(winInfo, constants_1.messageName, message);
        }
    };
    MultiWinStore.prototype.sendWinMsgByName = function (name, message) {
        var clientId = this.clientNameIdMap[name];
        if (clientId) {
            this.sendWinMsg(clientId, message);
        }
    };
    MultiWinStore.prototype.sendWinMsgByGroup = function (group, message) {
        var e_3, _a;
        try {
            for (var _b = __values(this.clientIds), _c = _b.next(); !_c.done; _c = _b.next()) {
                var clientId = _c.value;
                var groups = this.groupsMap[clientId];
                if (groups && groups.indexOf(group) !== -1) {
                    this.sendWinMsg(clientId, message);
                }
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
    };
    MultiWinStore.prototype.changeWinProps = function (clientId, winProps) { };
    MultiWinStore.prototype._genClientId = function () {
        var clientId = "win" + Math.floor(Math.random() * 10000);
        if (this.clientIds.indexOf(clientId) !== -1) {
            return this._genClientId();
        }
        return clientId;
    };
    MultiWinStore.prototype.closeAllWindows = function () {
        var _this = this;
        this.clientIds.slice(0).forEach(function (clientId) {
            _this.closeWin(clientId);
        });
    };
    return MultiWinStore;
}(event_flux_1.StoreBase));
exports.default = MultiWinStore;
