import { AppStore, AnyStoreDeclarer, DispatchItem } from "event-flux";
import { IRendererClientCallback, IRendererClient } from "./rendererClientTypes";
import { IWinProps } from "./mainClientTypes";
import { IStoreDispatcher, IDispatchInfo } from "./storeProxy/DispatchItemProxy";
import { Emitter } from "event-kit";
import WindowProxy from "./WindowProxy";
declare class IDGenerator {
    count: number;
    genID(): number;
}
declare global {
    interface Window {
        eventFluxWin: IWinProps;
        winId: string;
        parentId?: string;
        parentWin: WindowProxy;
    }
}
export interface IAppStoreOptions {
    serializer: (value: any) => string;
    deserializer: (str: string) => any;
    RendererClient?: IRendererClient;
}
export default class RendererAppStore extends AppStore implements IRendererClientCallback, IStoreDispatcher {
    rendererClient: IRendererClient;
    idGenerator: IDGenerator;
    resolveMap: {
        [invokeId: string]: {
            resolve: (data: any) => void;
            reject: (err: any) => void;
        };
    };
    mainInvokeMap: {
        [invokeId: string]: (...args: any[]) => void;
    };
    winId: string;
    emitter: Emitter<{
        [key: string]: any;
    }, {}>;
    serializer: (value: any) => string;
    deserializer: (str: string) => any;
    constructor(storeDeclarers?: AnyStoreDeclarer[], options?: IAppStoreOptions);
    handleDispatchReturn(actionStr: any): void;
    handleInvokeReturn(invokeId: string, error: any, result: any): void;
    handleMainInvoke(invokeId: string, args: any[]): void;
    handleMessage(data: any): void;
    handleWinMessage(senderId: string, data: any): void;
    handleInit(data: IWinProps): void;
    onDidMessage(callback: (data: any) => void): import("event-kit").Disposable;
    onDidWinMessage(callback: (params: {
        senderId: string;
        data: any;
    }) => void): import("event-kit").Disposable;
    onDidInit(callback: (params: any) => void): import("event-kit").Disposable;
    handleMainRequestStores(storeNames: string[]): void;
    handleMainReleaseStores(storeNames: string[]): void;
    handleMainMapRequestStores(storeName: string, mapKeys: string[]): void;
    handleMainMapReleaseStores(storeName: string, mapKeys: string[]): void;
    sendWindowMessage(sourceId: string, targetId: string, data: any): void;
    handleDispatch(dispatchInfo: IDispatchInfo): Promise<unknown>;
    handleDispatchNoReturn(dispatchInfo: IDispatchInfo): void;
    handleDispatchDisposable(dispatchInfo: IDispatchInfo, callback: (...args: any[]) => void): {
        dispose: () => void;
    };
    /**
     * Find the storeKey's dependencies that will need create in main process.
     * @param storeKey
     */
    findMainDepList(storeKey: string): string[];
    _createStoreAndInject(storeKey: string): string[];
    _disposeStoreAndDeps(storeKey: string, store: DispatchItem): void;
}
export {};
//# sourceMappingURL=RendererAppStore.d.ts.map