"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var MainAppStore_1 = __importDefault(require("../MainAppStore"));
var event_flux_1 = require("event-flux");
var constants_1 = require("../constants");
var StoreDeclarer_1 = require("../StoreDeclarer");
jest.useFakeTimers();
jest.mock("../MainClient", function () {
    var MyMainClient = /** @class */ (function () {
        function MyMainClient() {
            this.sendWinMsg = jest.fn();
            this.activeWin = jest.fn();
            this.createWin = jest.fn();
            this.changeWin = jest.fn();
            this.closeWin = jest.fn();
        }
        return MyMainClient;
    }());
    return MyMainClient;
});
describe("MainAppStore", function () {
    test("should create successfully", function () {
        var mainAppStore = new MainAppStore_1.default([event_flux_1.declareStore(event_flux_1.StoreBase, [], { stateKey: "todo", storeKey: "todoStore" })]);
        mainAppStore.init();
        expect(mainAppStore.outStoreDeclarers).toEqual(JSON.stringify([
            {
                storeKey: "todoStore",
                stateKey: "todo",
                storeType: "Item",
                depStoreNames: [],
                _evs: ["onDidUpdate", "onWillUpdate", "observe"],
            },
        ]));
    });
    test("add win and request store should behave normally", function () {
        var mainAppStore = new MainAppStore_1.default([
            event_flux_1.declareStore(event_flux_1.StoreBase, [], { stateKey: "todo1", storeKey: "todo1Store" }),
            event_flux_1.declareStore(event_flux_1.StoreBase, ["todo1Store"], { stateKey: "todo2", storeKey: "todo2Store" }),
        ]);
        mainAppStore.init();
        mainAppStore.multiWinSaver.addWin({ winId: "win1" });
        expect(mainAppStore.winFilters.win1).toEqual({});
        expect(mainAppStore.winHoldStores.win1).toEqual(new Set());
        mainAppStore.handleRequestStores("win1", ["todo2Store"]);
        expect(mainAppStore.winHoldStores.win1).toEqual(new Set(["todo2Store"]));
        expect(mainAppStore.winFilters.win1).toEqual({ todo1: true, todo2: true });
        jest.runAllTimers();
        expect(mainAppStore.mainClient.sendWinMsg).toHaveBeenLastCalledWith({ winId: "win1" }, constants_1.mainDispatchName, JSON.stringify({ updated: { todo2: {}, todo1: {} }, deleted: {} }));
        mainAppStore.handleReleaseStores("win1", ["todo2Store"]);
        expect(mainAppStore.winHoldStores.win1).toEqual(new Set());
        expect(mainAppStore.winFilters.win1).toEqual({});
        jest.runAllTimers();
        expect(mainAppStore.mainClient.sendWinMsg).toHaveBeenLastCalledWith({ winId: "win1" }, constants_1.mainDispatchName, JSON.stringify({ updated: {}, deleted: { todo2: true, todo1: true } }));
    });
    test("request store and release store for storeMap should behave normally", function () {
        var mainAppStore = new MainAppStore_1.default([
            event_flux_1.declareStore(event_flux_1.StoreBase, [], { stateKey: "todo1", storeKey: "todo1Store" }),
            event_flux_1.declareStoreMap(event_flux_1.StoreBase, ["todo1Store"], { stateKey: "todo2", storeKey: "todo2Store" }),
        ]);
        mainAppStore.setRecycleStrategy(event_flux_1.RecycleStrategy.Urgent);
        mainAppStore.init();
        mainAppStore.multiWinSaver.addWin({ winId: "win1" });
        mainAppStore.multiWinSaver.addWin({ winId: "win2" });
        expect(mainAppStore.winFilters.win1).toEqual({});
        expect(mainAppStore.winHoldStores.win1).toEqual(new Set());
        mainAppStore.handleRequestStores("win1", ["todo2Store"]);
        mainAppStore.handleRequestStores("win2", ["todo2Store"]);
        expect(mainAppStore.winHoldStores.win1).toEqual(new Set(["todo2Store"]));
        expect(mainAppStore.winFilters.win1).toEqual({ todo1: true, todo2: true });
        expect(mainAppStore.winFilters.win2).toEqual({ todo1: true, todo2: true });
        mainAppStore.handleMapRequestStores("win1", "todo2Store", ["key1", "key2"]);
        expect(Array.from(mainAppStore.stores.todo2Store.storeMap.keys())).toEqual(["key1", "key2"]);
        expect(mainAppStore.winHoldStores.win1).toEqual(new Set(["todo2Store", "todo2Store@key1", "todo2Store@key2"]));
        expect(mainAppStore.winFilters.win1).toEqual({ todo1: true, todo2: { key1: true, key2: true } });
        expect(mainAppStore.winFilters.win2).toEqual({ todo1: true, todo2: {} });
        jest.runAllTimers();
        expect(mainAppStore.mainClient.sendWinMsg).toHaveBeenNthCalledWith(1, { winId: "win1" }, constants_1.mainDispatchName, JSON.stringify({ updated: { todo2: { key1: {}, key2: {} }, todo1: {} }, deleted: {} }));
        mainAppStore.mainClient.sendWinMsg.mockReset();
        // Request the todo2Store's key2 subStore for win2
        mainAppStore.handleMapRequestStores("win2", "todo2Store", ["key2"]);
        expect(mainAppStore.stores.todo2Store._keyRefs.key2).toBe(2);
        expect(mainAppStore.winHoldStores.win2).toEqual(new Set(["todo2Store", "todo2Store@key2"]));
        expect(mainAppStore.winFilters.win2).toEqual({ todo1: true, todo2: { key2: true } });
        jest.runAllTimers();
        expect(mainAppStore.mainClient.sendWinMsg).toHaveBeenCalledWith({ winId: "win2" }, constants_1.mainDispatchName, JSON.stringify({ updated: { todo2: { key2: {} } }, deleted: {} }));
        mainAppStore.multiWinSaver.addWin({ winId: "win3" });
        mainAppStore.handleRequestStores("win3", ["todo2Store"]);
        expect(mainAppStore.winFilters.win3).toEqual({ todo1: true, todo2: {} });
        mainAppStore.mainClient.sendWinMsg.mockReset();
        mainAppStore.handleMapReleaseStores("win1", "todo2Store", ["key1", "key2"]);
        expect(mainAppStore.winFilters.win1).toEqual({ todo1: true, todo2: {} });
        expect(mainAppStore.winHoldStores.win1).toEqual(new Set(["todo2Store"]));
        expect(Array.from(mainAppStore.stores.todo2Store.storeMap.keys())).toEqual(["key2"]);
        expect(mainAppStore.mainClient.sendWinMsg).toHaveBeenNthCalledWith(1, { winId: "win1" }, constants_1.mainDispatchName, JSON.stringify({ updated: {}, deleted: { todo2: { key1: true, key2: true } } }));
    });
    test("delete win should release all the stores in that win ", function () {
        var mainAppStore = new MainAppStore_1.default([
            event_flux_1.declareStore(event_flux_1.StoreBase, [], { stateKey: "todo1", storeKey: "todo1Store" }),
            event_flux_1.declareStoreMap(event_flux_1.StoreBase, ["todo1Store"], { stateKey: "todo2", storeKey: "todo2Store" }),
        ]);
        mainAppStore.setRecycleStrategy(event_flux_1.RecycleStrategy.Urgent);
        mainAppStore.init();
        mainAppStore.multiWinSaver.addWin({ winId: "win1" });
        mainAppStore.handleRequestStores("win1", ["todo2Store"]);
        mainAppStore.handleMapRequestStores("win1", "todo2Store", ["key1", "key2"]);
        mainAppStore.multiWinSaver.deleteWin("win1");
        expect(mainAppStore.winFilters).toEqual({});
        expect(mainAppStore.winHoldStores).toEqual({});
        expect(mainAppStore.stores).toEqual({});
    });
    test("requestStore and releaseStore can handle win specific stores", function () {
        var mainAppStore = new MainAppStore_1.default([
            event_flux_1.declareStore(event_flux_1.StoreBase, [], { stateKey: "todo2", storeKey: "todo2Store" }),
            StoreDeclarer_1.declareWinStore(event_flux_1.StoreBase, [], { stateKey: "todo1", storeKey: "todo1Store" }),
            StoreDeclarer_1.declareWinStore(event_flux_1.StoreBase, ["todo1Store", "todo2Store"], { stateKey: "todo3", storeKey: "todo3Store" }),
        ]);
        mainAppStore.setRecycleStrategy(event_flux_1.RecycleStrategy.Urgent);
        mainAppStore.init();
        mainAppStore.requestStore("todo3Store", "win1");
        expect(Object.keys(mainAppStore.stores).sort()).toEqual(["todo1Store@win1", "todo2Store", "todo3Store@win1"]);
        expect(mainAppStore.stores["todo1Store@win1"].winId).toBe("win1");
        expect(mainAppStore.state).toEqual({ "todo1@win1": {}, todo2: {}, "todo3@win1": {} });
        mainAppStore.releaseStore("todo3Store", "win1");
        expect(mainAppStore.stores).toEqual({});
        expect(mainAppStore.state).toEqual({ "todo1@win1": undefined, todo2: undefined, "todo3@win1": undefined });
    });
    test("transformWinState should transform win spec stateKey to plain key", function () {
        expect(MainAppStore_1.default.prototype._transformWinState({
            todo2: 12,
            "todo3@win1": 23,
        })).toEqual({ todo2: 12, todo3: 23 });
    });
    test("handleWillChange should forward the diff state to renderer process", function () {
        var mainAppStore = new MainAppStore_1.default([
            event_flux_1.declareStore(event_flux_1.StoreBase, [], { stateKey: "todo2", storeKey: "todo2Store" }),
            StoreDeclarer_1.declareWinStore(event_flux_1.StoreBase, [], { stateKey: "todo1", storeKey: "todo1Store" }),
            StoreDeclarer_1.declareWinStore(event_flux_1.StoreBase, ["todo1Store", "todo2Store"], { stateKey: "todo3", storeKey: "todo3Store" }),
        ]);
        mainAppStore.init();
        mainAppStore.multiWinSaver.addWin({ winId: "win1" });
        mainAppStore.handleRequestStores("win1", ["todo3Store"]);
        mainAppStore.handleWillChange({ todo2: { todo2: 2 } }, { todo2: { todo2: 3, todo3: 1 } });
        expect(mainAppStore.mainClient.sendWinMsg).toHaveBeenLastCalledWith({ winId: "win1" }, constants_1.mainDispatchName, JSON.stringify({
            updated: { todo2: { todo2: 3, todo3: 1 } },
            deleted: {},
        }));
        mainAppStore.handleWillChange({ todo1: 12, "todo1@win1": "win1" }, { todo1: 10, "todo1@win1": "win2" });
        expect(mainAppStore.mainClient.sendWinMsg).toHaveBeenLastCalledWith({ winId: "win1" }, constants_1.mainDispatchName, JSON.stringify({
            updated: { todo1: "win2" },
            deleted: {},
        }));
    });
    test("handleRendererDispatch should can dispatch the method invocation", function () { return __awaiter(_this, void 0, void 0, function () {
        var mainAppStore;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    mainAppStore = new MainAppStore_1.default([
                        event_flux_1.declareStore(event_flux_1.StoreBase, [], { stateKey: "todo2", storeKey: "todo2Store" }),
                        StoreDeclarer_1.declareWinStore(event_flux_1.StoreBase, [], { stateKey: "todo1", storeKey: "todo1Store" }),
                        StoreDeclarer_1.declareWinStore(event_flux_1.StoreBase, ["todo1Store", "todo2Store"], { stateKey: "todo3", storeKey: "todo3Store" }),
                    ]);
                    mainAppStore.init();
                    mainAppStore.multiWinSaver.addWin({ winId: "win1" });
                    mainAppStore.handleRequestStores("win1", ["todo3Store"]);
                    return [4 /*yield*/, mainAppStore.handleRendererDispatch("win1", "1", JSON.stringify({ store: "todo2Store", method: "getRefCount", args: [] }))];
                case 1:
                    _a.sent();
                    expect(mainAppStore.mainClient.sendWinMsg).toHaveBeenLastCalledWith({ winId: "win1" }, constants_1.mainReturnName, "1", undefined, "1");
                    mainAppStore.mainClient.sendWinMsg.mockReset();
                    return [4 /*yield*/, mainAppStore.handleRendererDispatch("win1", "2", JSON.stringify({ store: "todo1Store", method: "getRefCount", args: [] }))];
                case 2:
                    _a.sent();
                    expect(mainAppStore.mainClient.sendWinMsg).toHaveBeenLastCalledWith({ winId: "win1" }, constants_1.mainReturnName, "2", undefined, "1");
                    mainAppStore.mainClient.sendWinMsg.mockReset();
                    return [4 /*yield*/, mainAppStore.handleRendererDispatch("win1", "3", JSON.stringify({ store: "todo1Store", method: "none", args: [] }))];
                case 3:
                    _a.sent();
                    expect(mainAppStore.mainClient.sendWinMsg).toHaveBeenLastCalledWith({ winId: "win1" }, constants_1.mainReturnName, "3", { message: "The store todo1Store's method none is not exists", name: "Error" }, undefined);
                    return [2 /*return*/];
            }
        });
    }); });
    test("handleRendererDispatchObserve should observe and dispose store", function () { return __awaiter(_this, void 0, void 0, function () {
        var mainAppStore;
        return __generator(this, function (_a) {
            mainAppStore = new MainAppStore_1.default([event_flux_1.declareStore(event_flux_1.StoreBase, [], { stateKey: "todo2", storeKey: "todo2Store" })]);
            mainAppStore.init();
            mainAppStore.multiWinSaver.addWin({ winId: "win1" });
            mainAppStore.handleRequestStores("win1", ["todo2Store"]);
            expect(mainAppStore.winObserves).toEqual({ win1: {} });
            mainAppStore.handleRendererDispatchObserve("win1", "1", { store: "todo2Store", method: "observe" });
            expect(mainAppStore.winObserves).toEqual({ win1: { 1: mainAppStore.winObserves.win1["1"] } });
            expect(mainAppStore.mainClient.sendWinMsg).toHaveBeenLastCalledWith(mainAppStore.multiWinSaver.getWinInfo("win1"), constants_1.mainInvokeName, "1", [{}]);
            mainAppStore.stores.todo2Store.setState({ hello: "world" });
            expect(mainAppStore.mainClient.sendWinMsg).toHaveBeenLastCalledWith(mainAppStore.multiWinSaver.getWinInfo("win1"), constants_1.mainInvokeName, "1", [{ hello: "world" }]);
            mainAppStore.mainClient.sendWinMsg.mockReset();
            mainAppStore.handleRendererDispatchDispose("win1", "1");
            expect(mainAppStore.winObserves).toEqual({ win1: {} });
            mainAppStore.stores.todo2Store.setState({ hello: "next" });
            expect(mainAppStore.mainClient.sendWinMsg).not.toHaveBeenCalled();
            return [2 /*return*/];
        });
    }); });
});
