"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var MultiWinSaver_1 = __importDefault(require("../MultiWinSaver"));
describe("MultiWinSaver", function () {
    test("addWin and deleteWin", function () {
        var multiWinSaver = new MultiWinSaver_1.default();
        var handleAddWin = jest.fn();
        var handleDeleteWin = jest.fn();
        multiWinSaver.onDidAddWin(handleAddWin);
        multiWinSaver.onDidDeleteWin(handleDeleteWin);
        multiWinSaver.addWin({ winId: "win1", prop: "win1" });
        expect(multiWinSaver.winInfos).toEqual([{ winId: "win1", prop: "win1" }]);
        expect(multiWinSaver.winIdMap).toEqual({ win1: { winId: "win1", prop: "win1" } });
        expect(handleAddWin).toHaveBeenLastCalledWith("win1");
        multiWinSaver.deleteWin("win1");
        expect(multiWinSaver.winInfos).toEqual([]);
        expect(multiWinSaver.winIdMap).toEqual({});
        expect(handleDeleteWin).toHaveBeenLastCalledWith("win1");
    });
    test("registerWin only notify once.", function () {
        var multiWinSaver = new MultiWinSaver_1.default();
        var win1Register = jest.fn();
        multiWinSaver.whenRegister("win1", win1Register);
        multiWinSaver.registerWin("win1");
        var win2Register = jest.fn();
        multiWinSaver.whenRegister("win1", win2Register);
        expect(win1Register).toHaveBeenCalled();
        expect(win2Register).toHaveBeenCalled();
        win1Register.mockReset();
        multiWinSaver.registerWin("win1");
        expect(win1Register).not.toHaveBeenCalled();
    });
    test("only notify when the same win reigster", function () {
        var multiWinSaver = new MultiWinSaver_1.default();
        var win1Register = jest.fn();
        multiWinSaver.whenRegister("win1", win1Register);
        multiWinSaver.registerWin("win2");
        expect(win1Register).not.toHaveBeenCalled();
        var win2Register = jest.fn();
        multiWinSaver.whenRegister("win1", win2Register);
        expect(win2Register).not.toHaveBeenCalled();
    });
});
