"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var RendererAppStore_1 = __importDefault(require("../RendererAppStore"));
var StoreProxy_1 = require("../storeProxy/StoreProxy");
var event_flux_1 = require("event-flux");
var constants_1 = require("../constants");
jest.mock("../RendererClient", function () {
    var MyRenderClient = /** @class */ (function () {
        function MyRenderClient() {
            this.sendMainMsg = jest.fn();
        }
        MyRenderClient.prototype.getQuery = function () {
            return window.query;
        };
        return MyRenderClient;
    }());
    return MyRenderClient;
});
jest.useFakeTimers();
describe("RendererAppStore", function () {
    test("constructor with storeDeclares and init", function () {
        var mainDeclares = [
            { storeType: "Item", storeKey: "mainTodoStore", stateKey: "mainTodo", depStoreNames: [] },
        ];
        window.query = {
            winId: "win1",
            storeDeclarers: JSON.stringify(mainDeclares),
            state: JSON.stringify({ hello: 1 }),
        };
        var appStore = new RendererAppStore_1.default([event_flux_1.declareStore(event_flux_1.StoreBase, [], { storeKey: "todo1Store", stateKey: "todo1" })]);
        expect(appStore.state).toEqual({ hello: 1 });
        expect(Object.keys(appStore._storeRegisterMap).sort()).toEqual(["mainTodoStore", "todo1Store"]);
        expect(appStore._storeRegisterMap.mainTodoStore.Store).toBe(StoreProxy_1.StoreProxy);
        expect(appStore.rendererClient.sendMainMsg).toHaveBeenLastCalledWith(constants_1.renderRegisterName, "win1");
    });
    test("onDidMessage should observe messages", function () {
        window.query = {
            winId: "win1",
            parentId: "win2",
            storeDeclarers: JSON.stringify([]),
            state: JSON.stringify({ hello: 1 }),
        };
        var appStore = new RendererAppStore_1.default();
        var observer = jest.fn();
        appStore.onDidMessage(observer);
        appStore.handleMessage("hello");
        expect(observer).toHaveBeenCalledWith("hello");
        observer.mockReset();
        appStore.onDidWinMessage(observer);
        appStore.handleWinMessage("win2", "hello");
        expect(observer).toHaveBeenCalledWith({ senderId: "win2", data: "hello" });
        observer.mockReset();
        appStore.onDidInit(observer);
        appStore.handleInit({ parentId: "win2", path: "/hello" });
        expect(observer).toHaveBeenCalledWith({ parentId: "win2", path: "/hello" });
    });
    test("request and release stores that in the renderer process", function () {
        window.query = {
            winId: "win1",
            storeDeclarers: JSON.stringify([
                { storeType: "Item", storeKey: "mainTodo1Store", stateKey: "main1Todo", depStoreNames: [] },
                { storeType: "Item", storeKey: "mainTodo2Store", stateKey: "main2Todo", depStoreNames: ["mainTodo1Store"] },
            ]),
            state: JSON.stringify({ hello: 1 }),
        };
        var appStore = new RendererAppStore_1.default([
            event_flux_1.declareStore(event_flux_1.StoreBase, [], { storeKey: "todo1Store", stateKey: "todo1" }),
            event_flux_1.declareStore(event_flux_1.StoreBase, ["todo1Store"], { storeKey: "todo2Store", stateKey: "todo2" }),
            event_flux_1.declareStore(event_flux_1.StoreBase, ["mainTodo1Store"], { storeKey: "todo3Store", stateKey: "todo2" }),
        ]);
        appStore.setRecycleStrategy(event_flux_1.RecycleStrategy.Urgent);
        appStore.init();
        appStore.rendererClient.sendMainMsg.mockReset();
        appStore.requestStore("todo2Store");
        expect(Object.keys(appStore.stores).sort()).toEqual(["todo1Store", "todo2Store"]);
        appStore.releaseStore("todo2Store");
        expect(appStore.stores).toEqual({});
        expect(appStore.rendererClient.sendMainMsg).not.toHaveBeenCalled();
        appStore.rendererClient.sendMainMsg.mockReset();
        appStore.requestStore("mainTodo2Store");
        expect(Object.keys(appStore.stores).sort()).toEqual(["mainTodo1Store", "mainTodo2Store"]);
        expect(appStore.rendererClient.sendMainMsg).toHaveBeenLastCalledWith(constants_1.renderRequestStoreName, "win1", [
            "mainTodo2Store",
        ]);
        appStore.releaseStore("mainTodo2Store");
        expect(appStore.stores).toEqual({});
        expect(appStore.rendererClient.sendMainMsg).toHaveBeenLastCalledWith(constants_1.renderReleaseStoreName, "win1", [
            "mainTodo2Store",
        ]);
        appStore.rendererClient.sendMainMsg.mockReset();
        appStore.requestStore("todo3Store");
        expect(Object.keys(appStore.stores).sort()).toEqual(["mainTodo1Store", "todo3Store"]);
        appStore.releaseStore("todo3Store");
        expect(appStore.stores).toEqual({});
        expect(appStore.rendererClient.sendMainMsg).toHaveBeenLastCalledWith(constants_1.renderReleaseStoreName, "win1", [
            "mainTodo1Store",
        ]);
    });
    test("findMainDepList should can parse the dependencies", function () {
        window.query = {
            winId: "win1",
            storeDeclarers: JSON.stringify([]),
            state: JSON.stringify({ hello: 1 }),
        };
        var appStore = new RendererAppStore_1.default([
            event_flux_1.declareStore(event_flux_1.StoreBase, [], { storeKey: "todo1Store", stateKey: "todo1", forMain: true }),
            event_flux_1.declareStore(event_flux_1.StoreBase, ["todo1Store"], { storeKey: "todo2Store", stateKey: "todo2" }),
            event_flux_1.declareStore(event_flux_1.StoreBase, ["todo1Store", "todo2Store"], { storeKey: "todo3Store", stateKey: "todo3" }),
        ]);
        expect(appStore.findMainDepList("todo2Store")).toEqual(["todo1Store"]);
        expect(appStore.findMainDepList("todo3Store")).toEqual(["todo1Store"]);
        expect(appStore.findMainDepList("todo1Store")).toEqual(["todo1Store"]);
    });
    test("storeProxy should delegate the action to the mainAppStore", function () { return __awaiter(_this, void 0, void 0, function () {
        var appStore, mainStore, actionRes, _a, err_1, _b;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    window.query = {
                        winId: "win1",
                        storeDeclarers: JSON.stringify([
                            {
                                storeType: "Item",
                                storeKey: "mainTodo1Store",
                                stateKey: "main1Todo",
                                depStoreNames: [],
                                _invokers: ["doHello"],
                            },
                        ]),
                        state: JSON.stringify({ hello: 1 }),
                    };
                    appStore = new RendererAppStore_1.default([]);
                    appStore.init();
                    mainStore = appStore.requestStore("mainTodo1Store");
                    actionRes = mainStore.doHello("arg1", "arg2");
                    expect(appStore.rendererClient.sendMainMsg).toHaveBeenLastCalledWith(constants_1.renderDispatchName, "win1", 1, JSON.stringify({
                        store: "mainTodo1Store",
                        method: "doHello",
                        args: ["arg1", "arg2"],
                    }));
                    appStore.resolveMap[1].resolve("hello");
                    _a = expect;
                    return [4 /*yield*/, actionRes];
                case 1:
                    _a.apply(void 0, [_c.sent()]).toBe("hello");
                    actionRes = mainStore.doHello("arg1", "arg2");
                    appStore.resolveMap[2].reject("hello");
                    _c.label = 2;
                case 2:
                    _c.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, actionRes];
                case 3:
                    _c.sent();
                    return [3 /*break*/, 5];
                case 4:
                    err_1 = _c.sent();
                    expect(err_1).toBeTruthy();
                    return [3 /*break*/, 5];
                case 5:
                    actionRes = mainStore.doHello("arg1", "arg2");
                    appStore.handleInvokeReturn("3", undefined, JSON.stringify("result"));
                    _b = expect;
                    return [4 /*yield*/, actionRes];
                case 6:
                    _b.apply(void 0, [_c.sent()]).toBe("result");
                    return [2 /*return*/];
            }
        });
    }); });
    test("handleDispatchReturn should can merge the updated and deleted states", function () {
        window.query = {
            winId: "win1",
            storeDeclarers: JSON.stringify([]),
            state: JSON.stringify({ hello: 1, world: 2 }),
        };
        var appStore = new RendererAppStore_1.default([]);
        var originState = appStore.state;
        appStore.handleDispatchReturn(JSON.stringify({ updated: { hello: 3 }, deleted: { world: true } }));
        expect(appStore).not.toBe(originState);
        expect(appStore.state).toEqual({ hello: 3 });
    });
    test("handleDispatchDisposable should dispatch the observer to renderer app store", function () {
        window.query = {
            winId: "win1",
            storeDeclarers: JSON.stringify([
                {
                    storeType: "Item",
                    storeKey: "mainTodo1Store",
                    stateKey: "main1Todo",
                    depStoreNames: [],
                    _invokers: ["doHello"],
                    _evs: ["onDidUpdate", "observe"],
                },
            ]),
            state: JSON.stringify({ hello: 1, world: 2 }),
        };
        var appStore = new RendererAppStore_1.default([]);
        appStore.init();
        var observer = jest.fn();
        var disposable = appStore.handleDispatchDisposable({ store: "mainTodo1Store", method: "onDidUpdate" }, observer);
        expect(appStore.mainInvokeMap).toEqual({ 1: observer });
        appStore.handleMainInvoke("1", ["hello", "world"]);
        expect(observer).toHaveBeenLastCalledWith("hello", "world");
        disposable.dispose();
        expect(appStore.mainInvokeMap).toEqual({});
    });
});
