"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var WindowProxy_1 = __importDefault(require("../WindowProxy"));
var event_kit_1 = require("event-kit");
jest.useFakeTimers();
describe("WindowProxy", function () {
    test("should send and receive messages from winId", function () {
        var MyAppStore = /** @class */ (function () {
            function MyAppStore() {
                this.winId = "win1";
                this.emitter = new event_kit_1.Emitter();
                this.sendWindowMessage = jest.fn();
            }
            MyAppStore.prototype.onDidWinMessage = function (callback) {
                this.emitter.on("did-message", callback);
            };
            MyAppStore.prototype.sendMsg = function (data) {
                this.emitter.emit("did-message", data);
            };
            return MyAppStore;
        }());
        var windowProxy = new WindowProxy_1.default(new MyAppStore(), null);
        windowProxy.send("hello");
        expect(windowProxy.store.sendWindowMessage).not.toHaveBeenCalled();
        windowProxy.changeWinId("proxyWin");
        windowProxy.send("hello");
        expect(windowProxy.store.sendWindowMessage).toHaveBeenCalledWith("win1", "proxyWin", "hello");
        var observer = jest.fn();
        windowProxy.onDidReceiveMsg(observer);
        windowProxy.store.sendMsg({ senderId: "proxyWin", data: "world" });
        expect(observer).toHaveBeenCalledWith("world");
    });
});
