"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var constants_1 = require("../constants");
var queryHandler_1 = require("../utils/queryHandler");
function genBrowserUrl(url, winId, winProps) {
    if (url === void 0) { url = ""; }
    var genUrl = new URL(url, location.href);
    genUrl.search =
        "?" +
            queryHandler_1.encodeQuery(__assign({}, queryHandler_1.decodeQuery(genUrl.search.slice(1)), winProps, { winId: winId, isSlave: 1 }));
    return genUrl.toString();
}
var BrowserMainClient = /** @class */ (function () {
    function BrowserMainClient(multiWinSaver, callback, log) {
        this.multiWinSaver = multiWinSaver;
        this.mainClientCallback = callback;
        this.log = log;
        window.isMainClient = true;
        window.addEventListener("message", this._handleMessage.bind(this));
        this.multiWinSaver.addWin({
            winId: "mainClient",
            window: window,
        });
    }
    BrowserMainClient.prototype.sendWinMsg = function (winInfo, msgName) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        winInfo.window.postMessage({ action: msgName, data: args }, "*");
    };
    // closeAllWindows() {
    //   Object.keys(this.clients).forEach(clientId => {
    //     let window = this.clients[clientId];
    //     window && window.close();
    //   });
    // }
    BrowserMainClient.prototype.activeWin = function (winInfo) { };
    BrowserMainClient.prototype.createWin = function (winId, winProps, winParams) {
        var _this = this;
        var featureStr = Object.keys(winParams)
            .map(function (key) { return key + "=" + winParams[key]; })
            .join(",");
        var childWin = window.open(genBrowserUrl(winProps.path, winId, winProps), "newwindow", featureStr + ", toolbar=no, menubar=no, scrollbars=no, resizable=no, location=no, status=no, titlebar=no");
        if (childWin) {
            childWin.onbeforeunload = function () { return _this.multiWinSaver.deleteWin(winId); };
        }
        this.multiWinSaver.addWin({ winId: winId, window: childWin });
        return childWin;
    };
    BrowserMainClient.prototype.changeWin = function (winInfo, winProps, winParams) { };
    BrowserMainClient.prototype.closeWin = function (winInfo) {
        winInfo.window.close();
    };
    BrowserMainClient.prototype._handleMessage = function (event) {
        var _a = event.data || {}, action = _a.action, payload = _a.data;
        switch (action) {
            case constants_1.renderDispatchName: {
                var _b = __read(payload, 3), clientId = _b[0], invokeId = _b[1], args = _b[2];
                this.mainClientCallback.handleRendererDispatch(clientId, invokeId, args);
                break;
            }
            case constants_1.renderDispatchNoReturnName: {
                var _c = __read(payload, 2), clientId = _c[0], args = _c[1];
                this.mainClientCallback.handleRendererDispatchNoReturn(clientId, args);
                break;
            }
            case constants_1.renderDispatchObserve: {
                var _d = __read(payload, 3), clientId = _d[0], invokeId = _d[1], method = _d[2];
                this.mainClientCallback.handleRendererDispatchObserve(clientId, invokeId, method);
                break;
            }
            case constants_1.renderDispatchDispose: {
                var _e = __read(payload, 2), clientId = _e[0], invokeId = _e[1];
                this.mainClientCallback.handleRendererDispatchDispose(clientId, invokeId);
                break;
            }
            case constants_1.renderRequestStoreName: {
                var _f = __read(payload, 2), clientId = _f[0], storeKeys = _f[1];
                this.mainClientCallback.handleRequestStores(clientId, storeKeys);
                break;
            }
            case constants_1.renderReleaseStoreName: {
                var _g = __read(payload, 2), clientId = _g[0], storeKeys = _g[1];
                this.mainClientCallback.handleReleaseStores(clientId, storeKeys);
                break;
            }
            case constants_1.renderMapRequestStoreName: {
                var _h = __read(payload, 3), clientId = _h[0], storeKey = _h[1], mapKeys = _h[2];
                this.mainClientCallback.handleMapRequestStores(clientId, storeKey, mapKeys);
                break;
            }
            case constants_1.renderMapReleaseStoreName: {
                var _j = __read(payload, 3), clientId = _j[0], storeKey = _j[1], mapKeys = _j[2];
                this.mainClientCallback.handleMapReleaseStores(clientId, storeKey, mapKeys);
                break;
            }
            case constants_1.winMessageName: {
                var _k = __read(payload, 3), senderId = _k[0], clientId = _k[1], data = _k[2];
                this.mainClientCallback.handleWinMessage(senderId, clientId, data);
                break;
            }
            case "close": {
                // Child window has closed
                var _l = __read(payload, 1), clientId = _l[0];
                this.multiWinSaver.deleteWin(clientId);
                break;
            }
            case constants_1.renderRegisterName: {
                var _m = __read(payload, 1), clientId = _m[0];
                this.mainClientCallback.handleRegisterWin(clientId);
                break;
            }
        }
    };
    return BrowserMainClient;
}());
exports.default = BrowserMainClient;
