"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var constants_1 = require("../constants");
var electron_1 = require("electron");
var queryHandler_1 = require("../utils/queryHandler");
var ElectronMainClient = /** @class */ (function () {
    function ElectronMainClient(multiWinSaver, callback, log) {
        var _this = this;
        this.multiWinSaver = multiWinSaver;
        this.mainClientCallback = callback;
        this.log = log;
        electron_1.ipcMain.on(constants_1.renderDispatchName, function (event, clientId, invokeId, args) {
            _this.mainClientCallback.handleRendererDispatch(clientId, invokeId, args);
        });
        electron_1.ipcMain.on(constants_1.renderDispatchNoReturnName, function (event, clientId, args) {
            _this.mainClientCallback.handleRendererDispatchNoReturn(clientId, args);
        });
        electron_1.ipcMain.on(constants_1.renderDispatchObserve, function (event, clientId, invokeId, method) {
            _this.mainClientCallback.handleRendererDispatchObserve(clientId, invokeId, method);
        });
        electron_1.ipcMain.on(constants_1.renderDispatchDispose, function (event, clientId, invokeId) {
            _this.mainClientCallback.handleRendererDispatchDispose(clientId, invokeId);
        });
        electron_1.ipcMain.on(constants_1.renderRequestStoreName, function (event, clientId, storeKeys) {
            _this.mainClientCallback.handleRequestStores(clientId, storeKeys);
        });
        electron_1.ipcMain.on(constants_1.renderReleaseStoreName, function (event, clientId, storeKeys) {
            _this.mainClientCallback.handleReleaseStores(clientId, storeKeys);
        });
        electron_1.ipcMain.on(constants_1.renderMapRequestStoreName, function (event, clientId, storeKey, mapKeys) {
            _this.mainClientCallback.handleMapRequestStores(clientId, storeKey, mapKeys);
        });
        electron_1.ipcMain.on(constants_1.renderMapReleaseStoreName, function (event, clientId, storeKey, mapKeys) {
            _this.mainClientCallback.handleMapReleaseStores(clientId, storeKey, mapKeys);
        });
        electron_1.ipcMain.on(constants_1.winMessageName, function (event, senderId, clientId, data) {
            _this.mainClientCallback.handleWinMessage(senderId, clientId, data);
        });
        electron_1.ipcMain.on(constants_1.renderRegisterName, function (event, clientId) {
            _this.mainClientCallback.handleRegisterWin(clientId);
        });
    }
    ElectronMainClient.prototype.sendWinMsg = function (winInfo, msgName) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        this._sendForWebContents.apply(this, __spread([winInfo.webContents, msgName], args));
    };
    ElectronMainClient.prototype.activeWin = function (winInfo) {
        var win = winInfo.window;
        if (win) {
            win.moveTop();
            // win && win.minimize();
            win.focus();
        }
    };
    ElectronMainClient.prototype.createWin = function (winId, winProps, winParams) {
        var _this = this;
        var storeDeclarers = this.mainClientCallback.getStoreDeclarers();
        var initStates = this.mainClientCallback.getInitStates(winId);
        var window = new electron_1.BrowserWindow(__assign({ webPreferences: {
                nodeIntegration: true,
            } }, winParams, { show: false }));
        var query = __assign({ winId: winId, storeDeclarers: storeDeclarers, state: initStates }, winProps);
        if (process.env.NODE_ENV === "development") {
            window.loadURL("http://localhost:" + process.env.ELECTRON_WEBPACK_WDS_PORT + "?" + queryHandler_1.encodeQuery(query));
            window.webContents.openDevTools();
        }
        else {
            window.loadURL(require("url").format({
                pathname: require("path").join(__dirname, "index.html"),
                protocol: "file",
                slashes: true,
                query: query,
            }));
        }
        if (winParams.show || winParams.show === undefined) {
            window.on("ready-to-show", function () {
                window.show();
            });
        }
        window.on("closed", function () {
            _this.multiWinSaver.deleteWin(winId);
        });
        this.multiWinSaver.addWin({ winId: winId, window: window, webContents: window.webContents });
        return window;
    };
    ElectronMainClient.prototype.changeWin = function (winInfo, winProps, winParams) {
        this.mainClientCallback.initWin(winInfo.winId, winProps);
        var setBoundsFunc = winParams.useContentSize ? "setContentBounds" : "setBounds";
        var win = winInfo.window;
        var x = Math.floor(winParams.x || 0);
        var y = Math.floor(winParams.y || 0);
        var width = Math.floor(winParams.width || 800), height = Math.floor(winParams.height || 600);
        if (winParams.minWidth && winParams.minHeight) {
            win.setMinimumSize(Math.floor(winParams.minWidth), Math.floor(winParams.minHeight));
        }
        if (winParams.maxWidth && winParams.maxHeight) {
            win.setMaximumSize(Math.floor(winParams.maxWidth), Math.floor(winParams.maxHeight));
        }
        if (winParams.title) {
            win.setTitle(winParams.title);
        }
        win[setBoundsFunc]({ x: x, y: y, width: width, height: height });
        win[setBoundsFunc]({ x: x, y: y, width: width, height: height });
        setTimeout(function () {
            win[setBoundsFunc]({ x: x, y: y, width: width, height: height });
            if (winParams.show) {
                win.show();
            }
        }, 0);
    };
    ElectronMainClient.prototype.closeWin = function (winInfo) {
        winInfo.window.close();
    };
    ElectronMainClient.prototype._sendForWebContents = function (webContents, channel) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        if (!webContents.isDestroyed() && !webContents.isCrashed()) {
            webContents.send.apply(webContents, __spread([channel], args));
        }
    };
    return ElectronMainClient;
}());
exports.default = ElectronMainClient;
