"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var MultiWinStore_1 = __importDefault(require("../MultiWinStore"));
var ElectronWindowState_1 = __importDefault(require("./ElectronWindowState"));
var electron_1 = require("electron");
var WindowManager = /** @class */ (function () {
    function WindowManager(winHandler) {
        var _this = this;
        this.windows = [];
        this.winSize = 1;
        this.windows = [];
        this.winHandler = winHandler;
        electron_1.app.whenReady().then(function () { return _this.ensureWindows(); });
        // this.ensureWindows();
    }
    WindowManager.prototype.genClientId = function () {
        var clientId = "win" + Math.floor(Math.random() * 10000);
        return clientId;
    };
    WindowManager.prototype.createWin = function (clientId) {
        return this.winHandler.createNativeWindow(clientId, "empty", undefined, { show: false });
    };
    WindowManager.prototype.ensureWindows = function () {
        if (!this.windows) {
            return;
        }
        while (this.windows.length < this.winSize) {
            var clientId = this.genClientId();
            this.windows.push({ clientId: clientId, window: this.createWin(clientId) });
        }
    };
    WindowManager.prototype.getWin = function () {
        if (!this.windows) {
            return undefined;
        }
        var winInfo = this.windows.shift();
        this.ensureWindows();
        return winInfo;
    };
    WindowManager.prototype.dispose = function () {
        this.windows.forEach(function (_a) {
            var window = _a.window;
            if (!window.isDestroyed()) {
                window.close();
            }
        });
        this.windows = [];
    };
    return WindowManager;
}());
exports.WindowManager = WindowManager;
var MultiWinCacheStore = /** @class */ (function (_super) {
    __extends(MultiWinCacheStore, _super);
    function MultiWinCacheStore() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.clientInfoMap = {};
        _this.willQuit = false;
        return _this;
    }
    MultiWinCacheStore.prototype.init = function () {
        this.loadClients();
    };
    MultiWinCacheStore.prototype.loadClients = function () {
        var _this = this;
        this.willQuit = false;
        var clients = this.getStorage().get("clients");
        if (!clients || clients.length === 0) {
            clients = this.getDefaultClients();
        }
        if (!this.windowManager) {
            this.windowManager = new WindowManager(this);
        }
        electron_1.app.whenReady().then(function () {
            clients.forEach(function (item) {
                _this.createWinForClientId({ path: item.path, groups: item.groups, name: item.name, parentId: item.parentId }, item.winId, item.winState);
            });
        });
    };
    MultiWinCacheStore.prototype.saveClients = function () {
        var _this = this;
        if (this.willQuit) {
            return;
        }
        var clients = this.clientIds.map(function (id) { return (__assign({ winId: id }, _this.clientInfoMap[id], { name: _this.clientIdNameMap[id], groups: _this.groupsMap[id] })); });
        this.getStorage().set("clients", clients);
    };
    MultiWinCacheStore.prototype.getDefaultClients = function () {
        return [{ winId: "mainClient", path: "/", name: "mainClient", groups: ["main"], winState: { isMaximized: true } }];
    };
    MultiWinCacheStore.prototype.getStorage = function () {
        // console.error("You need inherit MultiWinCacheStore and implement getStorage");
        if (!this._storage) {
            var Storage_1 = require("../storage").default;
            this._storage = new Storage_1("1.0");
        }
        return this._storage;
    };
    MultiWinCacheStore.prototype._removeClientId = function (clientId) {
        _super.prototype._removeClientId.call(this, clientId);
        delete this.clientInfoMap[clientId];
    };
    MultiWinCacheStore.prototype.handleDidCloseWin = function (winId) { };
    MultiWinCacheStore.prototype.saveWinState = function (clientId, winState) {
        this.clientInfoMap[clientId].winState = winState;
        // this.saveClients();
    };
    MultiWinCacheStore.prototype.createWinForClientId = function (winProps, clientId, params) {
        return this._createElectronWin(winProps, clientId, params);
    };
    MultiWinCacheStore.prototype.createWin = function (winProps, params) {
        return this._createWinForElectron(winProps, params);
    };
    MultiWinCacheStore.prototype._createWinForElectron = function (winProps, params) {
        winProps = this._parseWinProps(winProps);
        var parentClientId = winProps.parentId;
        if (params && params.x == null && params.y == null) {
            params.width = params.width || 800;
            params.height = params.height || 600;
            if (parentClientId && this.multiWinSaver.getWinInfo(parentClientId)) {
                var window_1 = this.multiWinSaver.getWinInfo(parentClientId).window;
                // let window = (this._appStore as any).mainClient.getWindowByClientId(parentClientId);
                var bounds = params.useContentSize ? window_1.getContentBounds() : window_1.getBounds();
                params.x = Math.floor(bounds.x + bounds.width / 2 - params.width / 2);
                params.y = Math.floor(bounds.y + bounds.height / 2 - params.height / 2);
            }
            else {
                var screenSize = electron_1.screen.getPrimaryDisplay().size;
                params.x = Math.floor(screenSize.width / 2 - params.width / 2);
                params.y = Math.floor(screenSize.height / 2 - params.height / 2);
            }
        }
        var clientId = null;
        try {
            clientId = this._createElectronWin(winProps, null, params);
        }
        catch (err) {
            console.error(err, err.stack);
        }
        return clientId;
    };
    MultiWinCacheStore.prototype._createElectronWin = function (url, clientId, params) {
        var _this = this;
        if (clientId && this.clientIds.indexOf(clientId) !== -1) {
            return null;
        }
        var winProps = this._parseWinProps(url);
        var winState = new ElectronWindowState_1.default(null, params, null);
        var parentId = winProps.parentId;
        var winInfo = this._getElectronWinFromCache(winProps.path, clientId, parentId, winState.state);
        if (!clientId) {
            clientId = winInfo.clientId;
        }
        this._addWinProps(clientId, winProps);
        var win = winInfo.win;
        this.clientInfoMap[clientId] = __assign({ winId: clientId, winState: winState.state }, winProps);
        winState.onSave = function (state) {
            _this.saveWinState(clientId, state);
        };
        // this.clientStateMap[clientId] = winState.state;
        winState.manage(win);
        this.saveClients(); // Save clients into Storage
        win.on("closed", this.handleClosed.bind(this, clientId));
        return clientId;
    };
    MultiWinCacheStore.prototype._getElectronWinFromCache = function (url, clientId, parentId, params) {
        // return createMainWindow(url, clientId, params);
        var win;
        if (clientId) {
            win = this.createMainWindow(url, clientId, parentId, params);
        }
        else {
            var winInfo = this.windowManager.getWin();
            if (!winInfo) {
                clientId = this._genClientId();
                win = this.createMainWindow(url, clientId, parentId, params);
            }
            else {
                clientId = winInfo.clientId;
                win = winInfo.window;
                this.mainClient.changeWin(this.multiWinSaver.getWinInfo(clientId), { path: url, parentId: parentId }, params);
            }
        }
        return { clientId: clientId, win: win };
    };
    MultiWinCacheStore.prototype.createNativeWindow = function (clientId, url, parentId, params) {
        if (url === void 0) { url = "empty"; }
        if (params == null) {
            params = {
                x: 0,
                y: 0,
                width: 1280,
                height: 800,
                useContentSize: true,
                webPreferences: {
                    nodeIntegration: true,
                },
            };
        }
        return this.mainClient.createWin(clientId, { path: url, parentId: parentId }, params);
    };
    MultiWinCacheStore.prototype.createMainWindow = function (url, clientId, parentId, params) {
        if (params === void 0) { params = {}; }
        var window = this.createNativeWindow(clientId, url, parentId, params);
        window.on("ready-to-show", function () {
            window.show();
        });
        return window;
    };
    MultiWinCacheStore.prototype.handleClosed = function (clientId) {
        if (clientId === "mainClient") {
            this.saveClients();
            this.willQuit = true;
        }
        this.handleDidCloseWin && this.handleDidCloseWin(clientId);
        this._removeClientId(clientId);
        if (!this.willQuit) {
            this.saveClients();
        }
        if (clientId === "mainClient") {
            this.closeAllWindows();
        }
    };
    MultiWinCacheStore.prototype.changeWinProps = function (clientId, winProps) {
        this.clientInfoMap[clientId] = __assign({}, this.clientInfoMap[clientId], winProps);
        this.saveClients();
    };
    MultiWinCacheStore.prototype.closeAllWindows = function () {
        _super.prototype.closeAllWindows.call(this);
        // Dispose the windowManager to destroy the hide windows.
        if (this.windowManager) {
            this.windowManager.dispose();
            this.windowManager = undefined;
        }
    };
    return MultiWinCacheStore;
}(MultiWinStore_1.default));
exports.default = MultiWinCacheStore;
