"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var ElectronWindowState_1 = __importDefault(require("../ElectronWindowState"));
var electron_1 = require("electron");
var electron_2 = require("electron");
jest.useFakeTimers();
describe("ElectronWindowState", function () {
    test("should loadState correctly", function () {
        var originMatching = electron_1.screen.getDisplayMatching;
        electron_1.screen.getDisplayMatching = jest.fn().mockImplementation(function () {
            return { bounds: { x: 0, y: 0, width: 1200, height: 800 } };
        });
        var winState = new ElectronWindowState_1.default({}, { x: 1.2, y: 2.3, width: 2.5, height: 1.2, useContentSize: true }, null);
        expect(winState.state).toEqual({ x: 1, y: 2, width: 2, height: 1, useContentSize: true });
        winState = new ElectronWindowState_1.default({}, { x: 1.2, y: 2.3 }, null);
        expect(winState.state).toEqual({ x: 1, y: 2, width: 800, height: 600, useContentSize: false });
        winState = new ElectronWindowState_1.default({}, {}, null);
        expect(winState.state).toEqual({ x: 0, y: 0, width: 800, height: 600, useContentSize: false });
        electron_1.screen.getDisplayMatching = jest.fn().mockImplementation(function () {
            return { bounds: { x: 0, y: 0, width: 400, height: 300 } };
        });
        winState = new ElectronWindowState_1.default({}, {}, null);
        expect(winState.state).toEqual({ x: 0, y: 0, width: 400, height: 300, useContentSize: false });
        electron_1.screen.getDisplayMatching = jest.fn().mockImplementation(function () {
            return { bounds: { x: -2000, y: 0, width: 400, height: 300 } };
        });
        winState = new ElectronWindowState_1.default({}, {}, null);
        expect(winState.state).toEqual({ x: -2000, y: 0, width: 400, height: 300, useContentSize: false });
        electron_1.screen.getDisplayMatching = jest.fn().mockImplementation(function () {
            return { bounds: { x: -2000, y: 0, width: 1200, height: 1200 } };
        });
        winState = new ElectronWindowState_1.default({}, {}, null);
        expect(winState.state).toEqual({ x: -2000 + 200, y: 300, width: 800, height: 600, useContentSize: false });
        electron_1.screen.getDisplayMatching = originMatching;
    });
    test("should manage window", function () {
        var winState = new ElectronWindowState_1.default({}, { isMaximized: true, isFullScreen: true }, jest.fn());
        var window = new electron_2.BrowserWindow();
        winState.manage(window);
        expect(window.isMaximized()).toBeTruthy();
        expect(window.isFullScreen()).toBeTruthy();
        window.setBounds({ x: 0, y: 0, width: 400, height: 300 });
        window.emit("move", {});
        jest.runAllTimers();
        expect(winState.state).toEqual({
            x: 0,
            y: 0,
            width: 800,
            height: 600,
            useContentSize: false,
            isMaximized: true,
            isFullScreen: true,
        });
        window.unmaximize();
        window.setFullScreen(false);
        window.emit("move", {});
        jest.runAllTimers();
        expect(winState.state).toEqual({
            x: 0,
            y: 0,
            width: 400,
            height: 300,
            useContentSize: false,
            isMaximized: false,
            isFullScreen: false,
        });
        window.setBounds({ x: 100, y: 100, width: 400, height: 300 });
        window.close();
        jest.runAllTimers();
        expect(winState.state).toEqual({
            x: 100,
            y: 100,
            width: 400,
            height: 300,
            useContentSize: false,
            isMaximized: false,
            isFullScreen: false,
        });
        expect(window._eventsCount).toBe(0);
        expect(winState.onSave).toHaveBeenCalled();
    });
});
