"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Store = require("electron-store");
var ElectronStore = /** @class */ (function () {
    function ElectronStore(version, store, ns) {
        this.store = store || new Store();
        this.ns = ns;
        if (version) {
            var curVersion = this.get("version", null);
            if (version !== curVersion) {
                this.clear();
                this.set("version", version);
            }
        }
    }
    ElectronStore.prototype.getNSStore = function (ns) {
        ns = this.ns ? this.ns + "." + ns : ns;
        return new ElectronStore(null, this.store, ns);
    };
    ElectronStore.prototype.set = function (key, value) {
        if (!this.ns) {
            return this.store.set(key, value);
        }
        var ns = this.ns;
        if (typeof key === "object") {
            var newObj = {};
            for (var k in key) {
                newObj[ns + "." + k] = key[k];
            }
            return this.store.set(newObj);
        }
        this.store.set(ns + "." + key, value);
    };
    ElectronStore.prototype.get = function (key, defaultValue) {
        var ns = this.ns;
        if (!ns) {
            return this.store.get(key, defaultValue);
        }
        return this.store.get(ns + "." + key, defaultValue);
    };
    ElectronStore.prototype.delete = function (key) {
        var ns = this.ns;
        if (!ns) {
            return this.store.delete(key);
        }
        return this.store.delete(ns + "." + key);
    };
    ElectronStore.prototype.deleteAll = function () {
        var ns = this.ns;
        if (!ns) {
            return this.store.clear();
        }
        return this.store.delete(ns);
    };
    ElectronStore.prototype.clear = function () {
        this.store.clear();
    };
    return ElectronStore;
}());
exports.default = ElectronStore;
