"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var event_flux_1 = require("event-flux");
var DispatchItemProxy_1 = __importDefault(require("./DispatchItemProxy"));
var event_kit_1 = require("event-kit");
var StoreMapItemProxy = /** @class */ (function (_super) {
    __extends(StoreMapItemProxy, _super);
    function StoreMapItemProxy(appStore, storeKey, indexKey) {
        var _this = _super.call(this) || this;
        return new Proxy(_this, {
            get: function (target, property, receiver) {
                if (target[property] != null) {
                    return target[property];
                }
                return function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    return appStore.handleDispatch({ store: storeKey, index: indexKey, method: property, args: args });
                };
            },
        });
    }
    return StoreMapItemProxy;
}(DispatchItemProxy_1.default));
var StoreMapProxy = /** @class */ (function (_super) {
    __extends(StoreMapProxy, _super);
    function StoreMapProxy(appStore, storeKey) {
        var _this = _super.call(this) || this;
        _this.length = 0;
        _this.storeMap = new Map();
        _this._keyRefs = {};
        _this._disposables = new event_kit_1.CompositeDisposable();
        _this.operateModeSwitch = new event_flux_1.OperateModeSwitch();
        _this._appStore = appStore;
        _this._storeKey = storeKey;
        return _this;
    }
    StoreMapProxy.prototype._inject = function (StoreBuilder, stateKey, depStores, initState, options) {
        this._stateKey = stateKey;
        var keys = options.keys;
        if (keys) {
            this.add(keys);
        }
    };
    StoreMapProxy.prototype._invokeRemoteMethod = function (method) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        this._appStore.handleDispatchNoReturn({ store: this._storeKey, method: method, args: args });
    };
    StoreMapProxy.prototype.requestStore = function (storeKey) {
        this.operateModeSwitch.enterRefCountMode();
        if (this._keyRefs[storeKey]) {
            this._keyRefs[storeKey] += 1;
        }
        else {
            this._keyRefs[storeKey] = 1;
            // this.add(storeKey);
            this._appStore.handleMainMapRequestStores(this._storeKey, [storeKey]);
            if (!this.storeMap.has(storeKey)) {
                this.storeMap.set(storeKey, new StoreMapItemProxy(this._appStore, this._storeKey, storeKey));
            }
        }
    };
    StoreMapProxy.prototype.releaseStore = function (storeKey) {
        this._keyRefs[storeKey] -= 1;
        if (this._keyRefs[storeKey] === 0) {
            // this.delete(storeKey);
            this._appStore.handleMainMapReleaseStores(this._storeKey, [storeKey]);
            this.storeMap.delete(storeKey);
        }
    };
    StoreMapProxy.prototype.request = function (keys) {
        var e_1, _a;
        var _this = this;
        if (!Array.isArray(keys)) {
            keys = [keys];
        }
        try {
            for (var keys_1 = __values(keys), keys_1_1 = keys_1.next(); !keys_1_1.done; keys_1_1 = keys_1.next()) {
                var key = keys_1_1.value;
                this.requestStore(key);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (keys_1_1 && !keys_1_1.done && (_a = keys_1.return)) _a.call(keys_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        var disposable = {
            dispose: function () {
                var e_2, _a;
                try {
                    for (var keys_2 = __values(keys), keys_2_1 = keys_2.next(); !keys_2_1.done; keys_2_1 = keys_2.next()) {
                        var key = keys_2_1.value;
                        _this.releaseStore(key);
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (keys_2_1 && !keys_2_1.done && (_a = keys_2.return)) _a.call(keys_2);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            },
        };
        this._disposables.add(disposable);
        return disposable;
    };
    StoreMapProxy.prototype.add = function (keys) {
        var _this = this;
        this.operateModeSwitch.enterDirectMode();
        this._appStore.handleDispatchNoReturn({ store: this._storeKey, method: "add", args: [keys] });
        var addOne = function (key) {
            if (_this.storeMap.has(key)) {
                return;
            }
            _this.storeMap.set(key, new StoreMapItemProxy(_this._appStore, _this._storeKey, key));
        };
        if (Array.isArray(keys)) {
            keys.forEach(function (key) { return addOne(key); });
        }
        else {
            addOne(keys);
        }
    };
    StoreMapProxy.prototype.delete = function (keys) {
        var _this = this;
        this._appStore.handleDispatchNoReturn({ store: this._storeKey, method: "delete", args: [keys] });
        if (Array.isArray(keys)) {
            keys.forEach(function (key) { return _this.storeMap.delete(key); });
        }
        else {
            this.storeMap.delete(keys);
        }
    };
    StoreMapProxy.prototype.get = function (index) {
        return this.storeMap.get(index);
    };
    StoreMapProxy.prototype.dispose = function () {
        this.storeMap.clear();
        this._disposables.dispose();
    };
    return StoreMapProxy;
}(DispatchItemProxy_1.default));
exports.StoreMapProxy = StoreMapProxy;
var StoreMapProxyDeclarer = /** @class */ (function (_super) {
    __extends(StoreMapProxyDeclarer, _super);
    function StoreMapProxyDeclarer() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    StoreMapProxyDeclarer.prototype.create = function (appStore) {
        var storeMapProxy = new StoreMapProxy(appStore, this.options.storeKey);
        return storeMapProxy;
    };
    return StoreMapProxyDeclarer;
}(event_flux_1.StoreMapDeclarer));
exports.StoreMapProxyDeclarer = StoreMapProxyDeclarer;
