"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var StoreMapProxy_1 = require("../StoreMapProxy");
var event_flux_1 = require("event-flux");
describe("StoreMapProxy", function () {
    test("should handle initial keys options", function () {
        var storeDispatcher = {
            handleDispatch: jest.fn(),
            handleDispatchNoReturn: jest.fn(),
            handleDispatchDisposable: jest.fn(),
        };
        var newStore = new StoreMapProxy_1.StoreMapProxy(storeDispatcher, "helloStore");
        newStore._inject(event_flux_1.StoreBase, "hello", {}, undefined, { keys: ["key1"] });
        expect(storeDispatcher.handleDispatchNoReturn).toHaveBeenLastCalledWith({
            store: "helloStore",
            method: "add",
            args: [["key1"]],
        });
    });
    test("should add and delete store proxy", function () {
        var storeDispatcher = {
            handleDispatch: jest.fn(),
            handleDispatchNoReturn: jest.fn(),
            handleDispatchDisposable: jest.fn(),
        };
        var newStore = new StoreMapProxy_1.StoreMapProxy(storeDispatcher, "helloStore");
        newStore.add("key1");
        expect(newStore.operateModeSwitch.operateMode).toEqual(event_flux_1.OperateMode.Direct);
        expect(storeDispatcher.handleDispatchNoReturn).toHaveBeenLastCalledWith({
            store: "helloStore",
            method: "add",
            args: ["key1"],
        });
        expect(Array.from(newStore.storeMap.keys())).toEqual(["key1"]);
        newStore.add(["key1", "key2", "key3"]);
        expect(storeDispatcher.handleDispatchNoReturn).toHaveBeenLastCalledWith({
            store: "helloStore",
            method: "add",
            args: [["key1", "key2", "key3"]],
        });
        expect(Array.from(newStore.storeMap.keys())).toEqual(["key1", "key2", "key3"]);
        newStore.delete("key1");
        expect(storeDispatcher.handleDispatchNoReturn).toHaveBeenLastCalledWith({
            store: "helloStore",
            method: "delete",
            args: ["key1"],
        });
        expect(Array.from(newStore.storeMap.keys())).toEqual(["key2", "key3"]);
        newStore.delete(["key1"]);
        expect(storeDispatcher.handleDispatchNoReturn).toHaveBeenLastCalledWith({
            store: "helloStore",
            method: "delete",
            args: [["key1"]],
        });
        expect(Array.from(newStore.storeMap.keys())).toEqual(["key2", "key3"]);
        expect(newStore.get("key2").doSome("hello", "do"));
        expect(storeDispatcher.handleDispatch).toHaveBeenLastCalledWith({
            store: "helloStore",
            index: "key2",
            method: "doSome",
            args: ["hello", "do"],
        });
        expect(function () { return newStore.request("key2"); }).toThrowError();
    });
    test("should requestStore and releaseStore store proxy", function () {
        var storeDispatcher = {
            handleDispatch: jest.fn(),
            handleDispatchNoReturn: jest.fn(),
            handleDispatchDisposable: jest.fn(),
            handleMainMapRequestStores: jest.fn(),
            handleMainMapReleaseStores: jest.fn(),
        };
        var newStore = new StoreMapProxy_1.StoreMapProxy(storeDispatcher, "helloStore");
        newStore.requestStore("key1");
        expect(storeDispatcher.handleMainMapRequestStores).toHaveBeenLastCalledWith("helloStore", ["key1"]);
        storeDispatcher.handleMainMapRequestStores.mockReset();
        newStore.requestStore("key1");
        expect(storeDispatcher.handleMainMapRequestStores).not.toHaveBeenCalled();
        expect(newStore._keyRefs.key1).toEqual(2);
        newStore.releaseStore("key1");
        expect(storeDispatcher.handleMainMapRequestStores).not.toHaveBeenCalled();
        newStore.releaseStore("key1");
        expect(storeDispatcher.handleMainMapReleaseStores).toHaveBeenLastCalledWith("helloStore", ["key1"]);
        expect(Array.from(newStore.storeMap.keys())).toEqual([]);
        var disposable1 = newStore.request("key1");
        expect(Array.from(newStore.storeMap.keys())).toEqual(["key1"]);
        var disposable2 = newStore.request("key1");
        expect(Array.from(newStore.storeMap.keys())).toEqual(["key1"]);
        disposable1.dispose();
        expect(Array.from(newStore.storeMap.keys())).toEqual(["key1"]);
        disposable2.dispose();
        expect(Array.from(newStore.storeMap.keys())).toEqual([]);
        expect(function () { return newStore.add("key2"); }).toThrowError();
    });
});
