"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var objUtils_1 = require("./objUtils");
var ListenForwardKeys = ["listen", "unlisten"];
var IndexForwardKeys = ["listen", "unlisten", "listenForKeys", "unlistenForKeys"];
var StoreProxyHandler = /** @class */ (function () {
    function StoreProxyHandler() {
        this.storeProxyCache = new Map();
    }
    /* Gen proxy for the storePath such as [todoMapStore, todoStore]
      forwardStore standard for the forward path
    */
    StoreProxyHandler.prototype.genProxy = function (storePath, forwardStore, forwarder) {
        return new Proxy(forwarder, {
            get: function (target, propName) {
                if (!propName) {
                    return;
                }
                if (forwardStore && forwardStore[propName]) {
                    return forwardStore[propName];
                }
                if (ListenForwardKeys.indexOf(propName) !== -1) {
                    return function () {
                        var args = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            args[_i] = arguments[_i];
                        }
                        args.unshift(window.clientId);
                        return forwarder({ store: storePath, method: propName, args: args });
                    };
                }
                return function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    return forwarder({ store: storePath, method: propName, args: args });
                };
            },
        });
    };
    StoreProxyHandler.prototype.genIndexProxy = function (storePath, childFilters, forwarder) {
        var _this = this;
        var storePathKey = storePath.slice(0, storePath.length - 1).toString();
        var newProxy = new Proxy(forwarder, {
            get: function (target, propName) {
                if (!propName) {
                    return;
                }
                var retIndexFunc = function (index) {
                    var cacheStore = _this.storeProxyCache.get(storePathKey + index);
                    if (cacheStore) {
                        return cacheStore;
                    }
                    var mapStorepath = __spread(storePath.slice(0, -1), [
                        __assign({}, storePath[storePath.length - 1], { index: index }),
                    ]);
                    var childStores = _this.proxyStore(mapStorepath, childFilters, forwarder);
                    var newStore = _this.genProxy(mapStorepath, childStores, forwarder);
                    _this.storeProxyCache.set(storePathKey + index, newStore);
                    return newStore;
                };
                if (IndexForwardKeys.indexOf(propName) !== -1) {
                    var oneStorePath_1 = __spread(storePath.slice(0, -1), [storePath[storePath.length - 1].name]);
                    return function () {
                        var args = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            args[_i] = arguments[_i];
                        }
                        args.unshift(window.clientId);
                        return forwarder({ store: oneStorePath_1, method: propName, args: args });
                    };
                }
                if (propName === "get") {
                    return retIndexFunc;
                }
                return retIndexFunc(propName);
            },
        });
        return newProxy;
    };
    StoreProxyHandler.prototype.proxyStore = function (parentStore, storeFilters, forwarder) {
        if (objUtils_1.isEmpty(storeFilters)) {
            return null;
        }
        var stores = {};
        for (var name_1 in storeFilters) {
            var storeInfo = storeFilters[name_1];
            if (storeInfo) {
                var type = storeInfo.type, filters = storeInfo.filters, _a = storeInfo.path, path = _a === void 0 ? parentStore : _a;
                var names = [];
                if (type === "Store") {
                    names = __spread(path, [name_1]);
                }
                else if (type === "StoreList") {
                    names = __spread(path, [{ name: name_1, type: "List" }]);
                }
                else if (type === "StoreMap") {
                    names = __spread(path, [{ name: name_1, type: "Map" }]);
                }
                if (type === "Store") {
                    var childStores = this.proxyStore(names, filters, forwarder);
                    stores[name_1] = this.genProxy(names, childStores, forwarder);
                }
                else {
                    stores[name_1] = this.genIndexProxy(names, filters, forwarder);
                }
            }
        }
        return stores;
    };
    StoreProxyHandler.prototype.proxyStores = function (storeFilters, forwarder) {
        return this.proxyStore([], storeFilters, forwarder);
    };
    return StoreProxyHandler;
}());
exports.default = StoreProxyHandler;
