"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var objectDifference_1 = __importDefault(require("../objectDifference"));
var immutable_1 = require("immutable");
test("base diff", function () {
    var sameObj = { hello: "a" };
    var oldObj = { d: "b", same: sameObj };
    var newObj = { m: "b", same: sameObj };
    var retObj = objectDifference_1.default(oldObj, newObj);
    expect(retObj).toEqual({
        updated: { m: "b" },
        deleted: { d: true },
    });
});
test("object diff with undefined value", function () {
    var sameObj = { hello: "a" };
    var oldObj = { d: "b", same: sameObj, v: undefined, e: 2 };
    var newObj = { m: "b", same: sameObj, v: 1, e: undefined };
    var retObj = objectDifference_1.default(oldObj, newObj);
    expect(retObj).toEqual({
        updated: { m: "b", v: 1, e: undefined },
        deleted: { d: true },
    });
});
test("immutable diff", function () {
    var _a, _b;
    var val1 = immutable_1.Map({ a: 2, b: 3 });
    var val2 = immutable_1.Map({ a: 3, c: 4 });
    val2 = val2.set({ key: "key" }, { value: "value" });
    var retObj = objectDifference_1.default(val1, val2);
    expect(retObj).toEqual({
        updated: (_a = {},
            _a[JSON.stringify("a")] = 3,
            _a[JSON.stringify("c")] = 4,
            _a[JSON.stringify({ key: "key" })] = { value: "value" },
            _a),
        deleted: (_b = {}, _b[JSON.stringify("b")] = true, _b),
    });
});
test("immutable and not immutable diff", function () {
    var val1 = { a: immutable_1.Map({ a: 2, b: 3 }) };
    var val2 = { a: 2, b: 3 };
    var retObj = objectDifference_1.default(val1, val2);
    expect(retObj).toEqual({
        updated: {
            a: 2,
            b: 3,
        },
        deleted: {},
    });
});
