"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var objectDifference_1 = __importDefault(require("../objectDifference"));
var objectMerge_1 = __importDefault(require("../objectMerge"));
var immutable_1 = require("immutable");
test("base merge", function () {
    var sameObj = { hello: "a" };
    var oldObj = { d: "b", same: sameObj };
    var newObj = { m: "b", same: sameObj };
    var _a = objectDifference_1.default(oldObj, newObj), updated = _a.updated, deleted = _a.deleted;
    var retObj = objectMerge_1.default(oldObj, updated, deleted);
    expect(retObj).toEqual(newObj);
});
test("base merge with undefined value", function () {
    var sameObj = { hello: "a" };
    var oldObj = { d: "b", same: sameObj, p: 1 };
    var newObj = { m: "b", same: sameObj, p: undefined };
    var _a = objectDifference_1.default(oldObj, newObj), updated = _a.updated, deleted = _a.deleted;
    var retObj = objectMerge_1.default(oldObj, updated, deleted);
    expect(retObj).toEqual(newObj);
});
test("base merge with undefined immutable value", function () {
    var _a, _b, _c, _d;
    var sameObj = { hello: "a" };
    var oldObj = immutable_1.fromJS({ d: "b", same: sameObj, same2: sameObj, p: 1 });
    var newObj = immutable_1.fromJS({ m: "b", same: {}, same2: { hello: "b" }, p: undefined });
    var _e = objectDifference_1.default(oldObj, newObj), updated = _e.updated, deleted = _e.deleted;
    expect(updated).toEqual((_a = {},
        _a[JSON.stringify("m")] = "b",
        _a[JSON.stringify("p")] = undefined,
        _a[JSON.stringify("same2")] = (_b = {}, _b[JSON.stringify("hello")] = "b", _b),
        _a));
    expect(deleted).toEqual((_c = {},
        _c[JSON.stringify("d")] = true,
        _c[JSON.stringify("same")] = (_d = {}, _d[JSON.stringify("hello")] = true, _d),
        _c));
    var retObj = objectMerge_1.default(oldObj, updated, deleted);
    expect(retObj).toEqual(newObj);
});
test("immutable diff", function () {
    var val1 = immutable_1.Map({ a: 2, b: 3, c: 4 });
    var val2 = immutable_1.Map({ a: 3, c: 4 });
    val2 = val2.set(10, 10);
    var _a = objectDifference_1.default(val1, val2), updated = _a.updated, deleted = _a.deleted;
    var retObj = objectMerge_1.default(val1, updated, deleted);
    expect(retObj.get("b")).toBeUndefined();
    expect(immutable_1.is(retObj, val2)).toBeTruthy();
    expect(retObj.get(10)).toBe(10);
});
test("immutable diff 2", function () {
    var oldPortfolio = {
        portfolioChartInfo: {
            t_913243980: immutable_1.fromJS({
                useIndicatorNames: [],
                compareTickers: [],
                interval: "m5K",
                selectLineType: null,
                tickerSettings: {},
                timespan: "Day5",
                chartType: "candles",
                useShape: "candle",
            }),
        },
    };
    var newPortfolio = {
        portfolioChartInfo: {
            t_913243980: immutable_1.fromJS({
                useIndicatorNames: [],
                compareTickers: [],
                selectLineType: null,
                tickerSettings: {},
                chartType: "candles",
                useShape: "candle",
            }),
        },
    };
    var _a = objectDifference_1.default(oldPortfolio, newPortfolio), updated = _a.updated, deleted = _a.deleted;
    var retObj = objectMerge_1.default(oldPortfolio, updated, deleted);
    expect(immutable_1.is(retObj.portfolioChartInfo.t_913243980, newPortfolio.portfolioChartInfo.t_913243980)).toBeTruthy();
});
