"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var StoreProxyHandler_1 = __importDefault(require("../StoreProxyHandler"));
test("basic", function () {
    var storeFilters = {
        store1: {
            type: "Store",
            filters: null,
            path: ["path1"],
        },
    };
    var mockFn = jest.fn();
    var stores = new StoreProxyHandler_1.default().proxyStores(storeFilters, mockFn);
    stores.store1.hello("arg1");
    expect(mockFn.mock.calls[0]).toEqual([
        {
            store: ["path1", "store1"],
            method: "hello",
            args: ["arg1"],
        },
    ]);
});
test("nest", function () {
    var storeFilters = {
        store1: {
            type: "Store",
            filters: {
                store2: {
                    type: "Store",
                    filters: null,
                },
            },
            path: ["path1"],
        },
    };
    var mockFn = jest.fn();
    var stores = new StoreProxyHandler_1.default().proxyStores(storeFilters, mockFn);
    stores.store1.store2.hello("world");
    expect(mockFn.mock.calls[0]).toEqual([
        {
            store: ["path1", "store1", "store2"],
            method: "hello",
            args: ["world"],
        },
    ]);
});
test("store map", function () {
    var storeFilters = {
        store1: {
            type: "StoreMap",
            filters: {
                store2: {
                    type: "Store",
                    filters: null,
                },
            },
            path: ["path1"],
        },
    };
    var mockFn = jest.fn();
    var stores = new StoreProxyHandler_1.default().proxyStores(storeFilters, mockFn);
    stores.store1.get("Key").store2.hello("world");
    expect(mockFn.mock.calls[0]).toEqual([
        {
            store: ["path1", { name: "store1", type: "Map", index: "Key" }, "store2"],
            method: "hello",
            args: ["world"],
        },
    ]);
    // Expect get the same store when map key is the same
    expect(stores.store1.get("Key")).toBe(stores.store1.get("Key"));
});
test("store map compare", function () {
    var storeFilters = {
        store1: {
            type: "StoreMap",
            filters: {
                store2: {
                    type: "Store",
                    filters: null,
                },
            },
            path: ["path1"],
        },
        store3: {
            type: "StoreMap",
            filters: {
                store2: {
                    type: "Store",
                    filters: null,
                },
            },
            path: [],
        },
    };
    var mockFn = jest.fn();
    var stores = new StoreProxyHandler_1.default().proxyStores(storeFilters, mockFn);
    expect(stores.store1.get("Key")).toBe(stores.store1.get("Key"));
    expect(stores.store1.get("Key")).not.toBe(stores.store3.get("Key"));
    expect(stores.store1.get("Key").store2).toBe(stores.store1.get("Key").store2);
});
