"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
  Apply the origin state with updated filter
  
  origin states are link { a: 1, b: 2, c: { c1: 'he', c2: 'dd' }}
  updated filter is like this: { b: true }

  ex:
  1. filterApply(
    { a: 1, b: 2, c: { c1: 'he', c2: 'dd' } },
    { c: true },
  ) would return
    { c: { c1: 'he', c2: 'dd' } }

  2. filterApply(
    { a: 1, b: { m: 'd' } },
    { b: true },
  ) would return
    { b: { m: 'd' } }

  3. filterApply(
    { a: { d: 1 }, b: { m: 'd' } },
    { a: true },
  ) would return
    { a: { d: 1 } }

  4. filterApply(
    { a: { d: 1 }, b: { m: 'd' } },
    { a: true },
  ) would return
    { a: { d: 1 } }
*/
var objUtils_1 = require("./objUtils");
function filterApply(origin, updated) {
    var merged = {};
    if (objUtils_1.isObject(updated)) {
        Object.keys(updated).forEach(function (key) {
            if (origin.hasOwnProperty(key) && updated[key]) {
                if (updated[key] === true) {
                    merged[key] = origin[key];
                    return;
                }
                merged[key] = filterApply(origin[key], updated[key]);
            }
        });
    }
    return merged;
}
exports.default = filterApply;
