"use strict";
/*
  Takes the old and the new version of an immutable object and
  returns a hash of what has updated (added or changed) in the object
  and what has been deleted in the object (with the entry that has
  been deleted given a value of true).

  ex:
  1. filterDifference({ m: true, a: 1 }, { m: false, b: 2 }) would return
    { updated: { m: false, b: 2 }, deleted: { a: true } }

  1. filterDifference({ m: true, b: 1 }, { m: false, b: 1 }) would return
    { updated: { m: false } }
*/
Object.defineProperty(exports, "__esModule", { value: true });
var objUtils_1 = require("./objUtils");
var isShallow = function (val) { return Array.isArray(val) || !objUtils_1.isObject(val); };
function checkUpdateVal(key, old, curr, updated, deleted) {
    var oldVal = old[key];
    var currVal = curr[key];
    if (currVal === oldVal) {
        return;
    }
    if (isShallow(currVal) || isShallow(oldVal)) {
        updated[key] = currVal;
    }
    else {
        var diff = filterDifference(oldVal, currVal);
        !objUtils_1.isEmpty(diff.updated) && (updated[key] = diff.updated);
        !objUtils_1.isEmpty(diff.deleted) && (deleted[key] = diff.deleted);
    }
}
function filterDifference(old, curr) {
    if (old == null || curr == null) {
        return { updated: curr, deleted: {} };
    }
    var updated = {};
    var deleted = {};
    Object.keys(curr).forEach(function (key) { return checkUpdateVal(key, old, curr, updated, deleted); });
    Object.keys(old).forEach(function (key) {
        if (!curr.hasOwnProperty(key)) {
            deleted[key] = true;
        }
    });
    return { updated: updated, deleted: deleted };
}
exports.default = filterDifference;
