"use strict";
/*
  Takes the old and the new version of an immutable object and
  returns a hash of what has updated (added or changed) in the object
  and what has been deleted in the object (with the entry that has
  been deleted given a value of true).

  ex: objectDifference({a: 1}, {b: 2}) would return
    {updated: {b: 2}, deleted: {a: true}}
*/
Object.defineProperty(exports, "__esModule", { value: true });
var immutable_1 = require("immutable");
var objUtils_1 = require("./objUtils");
var isShallow = function (val) { return Array.isArray(val) || !objUtils_1.isObject(val) || immutable_1.List.isList(val); };
var isDiffType = function (val1, val2) {
    return (immutable_1.Map.isMap(val1) && !immutable_1.Map.isMap(val2)) || (!immutable_1.Map.isMap(val1) && immutable_1.Map.isMap(val2));
};
function checkUpdateVal(key, oldVal, currVal, updated, deleted) {
    if (currVal === oldVal) {
        return;
    }
    if (isShallow(currVal) || isShallow(oldVal)) {
        updated[key] = currVal;
    }
    else if (isDiffType(currVal, oldVal)) {
        updated[key] = currVal;
    }
    else {
        var diff = objectDifference(oldVal, currVal);
        !objUtils_1.isEmpty(diff.updated) && (updated[key] = diff.updated);
        !objUtils_1.isEmpty(diff.deleted) && (deleted[key] = diff.deleted);
    }
}
function objectDifference(old, curr) {
    var updated = {};
    var deleted = {};
    if (immutable_1.Map.isMap(old) && immutable_1.Map.isMap(curr)) {
        curr.forEach(function (val, key) { return checkUpdateVal(JSON.stringify(key), old.get(key), val, updated, deleted); });
        old.forEach(function (val, key) {
            if (!curr.has(key)) {
                deleted[JSON.stringify(key)] = true;
            }
        });
    }
    else {
        Object.keys(curr).forEach(function (key) { return checkUpdateVal(key, old[key], curr[key], updated, deleted); });
        Object.keys(old).forEach(function (key) {
            if (!curr.hasOwnProperty(key)) {
                deleted[key] = true;
            }
        });
    }
    return { updated: updated, deleted: deleted };
}
exports.default = objectDifference;
